/*
 * include/asm-arm/arch-cxd4115/dma.h
 *
 * DMA definitions
 *
 * Copyright 2007,2008,2009,2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#include <asm/io.h>
#include <mach/hardware.h>
#include <mach/regs-dmac.h>

struct dma_lli {
	unsigned long src;
	unsigned long dst;
	unsigned long next;
	unsigned long ctrl;
};

int dma_stop(unsigned int ch);

static inline int dma_start(unsigned int ch, struct dma_lli *p, unsigned int cfg)
{
	int ret;

	ret = dma_stop(ch);
	if (unlikely(ret))
		return ret;

	writel(p->src, DMACCSRCADDR_(ch));
	writel(p->dst, DMACCDESTADDR_(ch));
	writel(p->next, DMACCLLI_(ch));
	writel(p->ctrl, DMACCCONTROL_(ch));
	writel(cfg, DMACCCONFIGURATION_(ch));

	return ret;
}

static inline void dma_setup(struct dma_lli *p,
			    unsigned long src, unsigned long dst,
			    unsigned long next, unsigned long ctrl)
{
	p->src	= src;
	p->dst	= dst;
	p->next	= next;
	p->ctrl	= ctrl;
}

static inline void dma_clear(unsigned int ch)
{
	writel(1 << ch, DMACINTTCCLR); /* INT clear */
}

static inline int dma_inttc_status(unsigned int ch)
{
	return !!(readl(DMACINTTCSTATUS) & (1 << ch));
}

#endif /* _ASM_ARCH_DMA_H */
