/*
 * arch/arm/mach-cxd4132/fiq.c
 *
 * FIQ driver
 *
 * Copyright 2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <asm/system.h>
#include <asm/io.h>
#include <mach/memory.h>
#include <mach/regs-misc.h>
#include <mach/regs-permisc.h>

/* Called from early_trap_init() */
void mach_trap_init(unsigned long vectors)
{
	memcpy((void *)vectors + FIQ_VEC_OFFS, fiq_vec, fiq_vec_end - fiq_vec);
	memset((void *)vectors + CXD4132_FIQ_REGSAVE, 0, CXD4132_FIQ_REG_ALL);
	memcpy((void *)vectors + CXD4132_FIQ_DEBUG, fiq_debug, fiq_debug_end - fiq_debug);
	/* cache flush will be done in early_trap_init() */
}

void cxd4132_fiq_init(void)
{
	unsigned long modepin;

	modepin = readl(VA_MISCCTRL(MISCCTRL_MODEREAD));
	if (modepin & MODEREAD_BM1) {
		printk(KERN_INFO "FIQ: disabled\n");
		return;
	}

	/* setup PER_FIQEN */
	writel(PER_FIQEN_BM1|PER_FIQEN_ALL, VA_PERMISC+PER_FIQEN);

	local_fiq_enable();
}
