/*
 * noncache.h
 *
 * Physical address to NonCacheable/Cacheable address converter
 * Virtual address to Physical address converter
 *
 * Copyright 2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#ifndef __MACH_CXD4132_INCLUDE_MACH_NONCACHE_H
#define __MACH_CXD4132_INCLUDE_MACH_NONCACHE_H
/*
 * NAME
 * 	PHYS_TO_NONCACHE, PHYS_TO_CACHE MACRO - Convert physical address to
 *	{noncacheable,cacheable} kernel address
 *
 * SYNOPSIS
 *	#include <mach/noncache.h>
 *	unsigned long PHYS_TO_NONCACHE(unsigned long);
 *	unsigned long PHYS_TO_CACHE(unsigned long);
 * INPUT
 *	Physical address of DDR, eSRAM.
 * RETURN VALUE
 *	PHYS_TO_NONCACHE: Noncacheable kernel address
 *	PHYS_TO_CACHE:    Cacheable kernel address
 *
 *	If input is out of range, PA2NC_ERR shall be returned
 *	when range check is enabled.
 *
 * OPTION FLAG
 *	PA2NC_NO_RANGE_CHECK : disable range check
 *
 * note
 *   Prefix PA2NC_ means Physical Address TO NonCacheable address.
 */
/*
 * NAME
 * 	VA_TO_PHYS - Convert virtual address to physical address
 *
 * SYNOPSIS
 *	#include <mach/noncache.h>
 *	unsigned long VA_TO_PHYS(unsigned long);
 * INPUT
 *	Kernel direct map address of DDR, eSRAM.
 * RETURN VALUE
 *	Physical address
 *
 *	If input is out of range, PA2NC_ERR shall be returned
 *	when range check is enabled.
 *
 * OPTION FLAG
 *	PA2NC_NO_RANGE_CHECK : disable range check
 */
#include <mach/noncache-common.h>

/*
 *  CXD4132
 *
 *   Physical     NonCacheable  Cacheable
 *  0x80000000 --> 0x80000000	0xc0000000  ;DDR
 *  0xa0000000 --> 0xa0000000	0xe0000000  ;eSRAM
 */
#define PA2NC_VIRT_BIT		0x40000000UL
#define PA2NC_P2C(x)		((x) | PA2NC_VIRT_BIT)
#define PA2NC_V2P(x)		((x) & ~PA2NC_VIRT_BIT)
#ifdef PA2NC_NO_RANGE_CHECK

#define PHYS_TO_NONCACHE(x)	(x)
#define PHYS_TO_CACHE(x)	PA2NC_P2C(x)
#define VA_TO_PHYS(x)		PA2NC_V2P(x)

#else /* PA2NC_NO_RANGE_CHECK */

#define PA2NC_MEM_START		(0x80000000UL)
#define PA2NC_MEM_END		(0xa1000000UL)
#define PA2NC_VALID(x)		(PA2NC_IN((x), MEM))

#define PHYS_TO_NONCACHE(x) 	(PA2NC_VALID(x) ? (x) : PA2NC_ERR)
#define PHYS_TO_CACHE(x) 	(PA2NC_VALID(x) ? PA2NC_P2C(x) : PA2NC_ERR)
#define VA_TO_PHYS(x)		(PA2NC_VALID(PA2NC_V2P(x)) ? \
					PA2NC_V2P(x) : PA2NC_ERR)

#endif /* PA2NC_NO_RANGE_CHECK */
#endif /* __MACH_CXD4132_INCLUDE_MACH_NONCACHE_H */
