/*
 * arch/arm/plat-cxd41xx/cfg.c
 *
 * config parameters
 *
 * Copyright 2009,2010 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <mach/moduleparam.h>
#include <mach/cxd41xx_cpuid.h>
#include <mach/cxd41xx_cfg.h>

/*
 * CPUID
 */
unsigned long cxd41xx_cpuid = CXD41XX_CPUID_UNDEF;

void __init __attribute__((weak)) cxd41xx_update_cpuid(void)
{
	/* none */
}

/* Override CPUID by command line parameter */
static int __init set_cpuid(char *str)
{
	get_option(&str, (int *)&cxd41xx_cpuid);
	printk(KERN_INFO "CPUID override: 0x%lx\n", cxd41xx_cpuid);
	cxd41xx_update_cpuid();
	return 0;
}
early_param("cpuid", set_cpuid);

/*
 * STABLE_RUN port
 */
unsigned long cxd41xx_stable_run = CXD41XX_PORT_UNDEF;
module_param_named(stable_run, cxd41xx_stable_run, port, S_IRUSR);

/*
 * XPOWER_OFF (XPWR_OFF)
 */
unsigned long cxd41xx_xpower_off = CXD41XX_PORT_UNDEF;
module_param_named(poff, cxd41xx_xpower_off, port, S_IRUSR);

/*
 * UART ports which is handled by tty driver.
 */
unsigned long cxd41xx_uart_bitmap = 0x1UL; /* bit0=UART0 */
module_param_named(uart, cxd41xx_uart_bitmap, ulong, S_IRUSR);

#if defined(CONFIG_SMC9118) || defined(CONFIG_SMSC911X)
/*
 * Ether IRQ
 */
unsigned long cxd41xx_ether_irq = CONFIG_CXD41XX_SMC911X_INT;
module_param_named(ether_irq, cxd41xx_ether_irq, ulong, S_IRUSR);
#endif /* CONFIG_SMC9118 || CONFIG_SMSC911X */

/*
 * Observe tick and localtimer for debug
 */
unsigned long cxd41xx_tick_timing = 0UL;
module_param_named(tick_timing, cxd41xx_tick_timing, ulong, S_IRUSR|S_IWUSR);

/*
 * SMODE
 */
unsigned long cxd41xx_smode_port = CXD41XX_PORT_UNDEF;
module_param_named(smode, cxd41xx_smode_port, port, S_IRUSR);
int cxd41xx_smode = -1;
EXPORT_SYMBOL(cxd41xx_smode);

/*
 * ABT_A1 IRQ
 */
int cxd41xx_abt1_irq = -1;
module_param_named(abt1_irq, cxd41xx_abt1_irq, int, S_IRUSR);

/*
 * ABT_A1 port
 */
unsigned long cxd41xx_abt1_port = CXD41XX_PORT_UNDEF;
module_param_named(abt1, cxd41xx_abt1_port, port, S_IRUSR);
