/*
 * arch/arm/plat-cxd41xx/kslog.c
 *
 * kernel simple debug logger
 *
 * Copyright 2011 Sony Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include <linux/module.h>
#include <asm/io.h>
#include <mach/time.h>
#include <mach/kslog.h>

static unsigned long kslog_addr = 0;
module_param_named(addr, kslog_addr, ulongH, S_IRUSR);
static unsigned long kslog_size = 0;
module_param_named(size, kslog_size, ulongH, S_IRUSR);

static DEFINE_RAW_SPINLOCK(kslog_lock);
static char *kslogp = NULL;
static int kslogrest = 0;

void kslog_init(void)
{
	kslogp = (char *)kslog_addr;
	kslogrest = kslog_size;
}

void vksprintf(const char *fmt, va_list args)
{
	ulong t;
	int len;

	if (!kslogrest)
		return;
	spin_lock(&kslog_lock);
	t = read_freerun();
	/* kslogrest should be greater than 0. */
	len = scnprintf(kslogp, kslogrest, "%lu:", mach_cycles_to_usecs(t));
	/* len: [0 .. kslogrest-1] */
	kslogp += len;
	kslogrest -= len;
	/* So, kslogrest is >= 1 */
	len = vscnprintf(kslogp, kslogrest, fmt, args);
	kslogp += len;
	kslogrest -= len;
	if (1 == kslogrest) { /* full ? */
		kslogrest = 0;
	}
	spin_unlock(&kslog_lock);
}

void ksprintf(const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	vksprintf(fmt, args);
	va_end(args);
}

void kslog_show(void)
{
	char *p, *s;

	if (!kslog_addr || !kslogp)
		return;
	for (p = (char *)kslog_addr; p < kslogp; p = s + 1) {
		for (s = p; s < kslogp; s++) {
			if ('\n' == *s)
				break;
		}
		if (s == kslogp)
			s--;
		*s = '\0';
		printk(p);
		printk(KERN_CONT "\n");
		p = s + 1;
	}
	printk(KERN_ERR "kslog:rest=%d\n", kslogrest);
}
